@@ -39,7 +39,7 @@
 #' @importFrom utils str
 #' @keywords internal
 pid_exists <- local({
-  os <- .Platform$OS.type
+  os <- .Platform[["OS.type"]]
 
   ## The value of tools::pskill() is incorrect in R (< 3.5.0).
   ## This was fixed in R (>= 3.5.0).
@@ -51,10 +51,7 @@ pid_exists <- local({
         ##  signal is actually sent. The null signal can be used to check the 
         ##  validity of pid." [1]
         res <- pskill(pid, signal = 0L)
-        if (debug) {
-          cat(sprintf("Call: tools::pskill(%s, signal = 0L)\n", pid))
-          print(res)
-        }
+        if (debug) mdebugf("Call: tools::pskill(%s, signal = 0L): %s", pid, res)
         as.logical(res)
       }, error = function(ex) NA)
     }
@@ -70,29 +67,25 @@ pid_exists <- local({
         system2("ps", args = pid, stdout = TRUE, stderr = FALSE)
       })
       if (debug) {
-        cat(sprintf("Call: ps %s\n", pid))
-        print(out)
-        str(out)
+        mdebugf("Call: ps %s", pid)
+        mprint(out)
+        mstr(out)
       }
       status <- attr(out, "status")
       if (is.numeric(status) && status < 0) return(NA)
       out <- gsub("(^[ ]+|[ ]+$)", "", out)
       out <- out[nzchar(out)]
       if (debug) {
-        cat("Trimmed:\n")
-        print(out)
-        str(out)
+        mdebug("Trimmed:")
+        mprint(out)
+        mstr(out)
       }
       out <- strsplit(out, split = "[ ]+", fixed = FALSE)
       out <- lapply(out, FUN = function(x) x[1])
       out <- unlist(out, use.names = FALSE)
-      if (debug) {
-        cat("Extracted: ", paste(sQuote(out), collapse = ", "), "\n", sep = "")
-      }
+      if (debug) mdebugf("Extracted: %s", commaq(out))
       out <- suppressWarnings(as.integer(out))
-      if (debug) {
-        cat("Parsed: ", paste(sQuote(out), collapse = ", "), "\n", sep = "")
-      }
+      if (debug) mdebugf("Parsed: %s", commaq(out))
       any(out == pid)
     }, error = function(ex) NA)
   }
@@ -107,21 +100,19 @@ pid_exists <- local({
         args = c("/FI", shQuote(sprintf("PID eq %.0f", pid)), "/NH")
         out <- system2("tasklist", args = args, stdout = TRUE, stderr = "")
         if (debug) {
-          cat(sprintf("Call: tasklist %s\n", paste(args, collapse = " ")))
-          print(out)
-          str(out)
+          mdebugf("Call: tasklist %s", paste(args, collapse = " "))
+          mprint(out)
+          mstr(out)
         }
         out <- gsub("(^[ ]+|[ ]+$)", "", out)
         out <- out[nzchar(out)]
         if (debug) {
-          cat("Trimmed:\n")
-          print(out)
-          str(out)
+          mdebug("Trimmed:")
+          mprint(out)
+          mstr(out)
         }
         out <- grepl(sprintf(" %.0f ", pid), out)
-        if (debug) {
-          cat("Contains PID: ", paste(out, collapse = ", "), "\n", sep = "")
-        }
+        if (debug) mdebugf("Contains PID: %s", commaq(out))
         any(out)
       }, error = function(ex) NA)
       if (isTRUE(res)) return(res)
@@ -136,42 +127,45 @@ pid_exists <- local({
       res <- tryCatch({
         out <- system2("tasklist", stdout = TRUE, stderr = "")
         if (debug) {
-          cat("Call: tasklist\n")
-          print(out)
-          str(out)
+          mdebug("Call: tasklist")
+          mprint(out)
+          mstr(out)
         }
         out <- gsub("(^[ ]+|[ ]+$)", "", out)
         out <- out[nzchar(out)]
         skip <- grep("^====", out)[1]
-        if (!is.na(skip)) out <- out[seq(from = skip + 1L, to = length(out))]
+        if (!is.na(skip)) {
+	  ## Parse the ===== bar to identify column widths
+	  bar <- out[skip]
+	  idxs <- which(strsplit(bar, split = "", fixed = TRUE)[[1]] == " ")
+	  from <- c(1L, idxs + 1L)
+	  to <- c(idxs-1L, nchar(bar))
+	  cols <- rbind(from, to)
+	  bar2 <- apply(cols, MARGIN = 2L, FUN = function(x) { substr(bar, start = x[1], stop = x[2]) })
+          if (debug) {
+            mdebug("Column widths:")
+            mprint(bar)
+            mprint(cols)
+            mprint(bar2)
+          }
+	  stop_if_not(all(grepl("^=+$", bar2)))
+	  out <- out[seq(from = skip + 1L, to = length(out))]
+	  out <- apply(cols, MARGIN = 2L, FUN = function(x) {
+	    value <- substr(out, start = x[1], stop = x[2])
+            gsub("(^[ ]+|[ ]+$)", "", value)
+	  })
+	}
         if (debug) {
-          cat("Trimmed:\n")
-          print(out)
-          str(out)
-        }
-        out <- strsplit(out, split = "[ ]+", fixed = FALSE)
-        ## WORKAROUND: The 'Image Name' column may contain spaces, making
-        ## it hard to locate the second column.  Instead, we will identify
-        ## the most common number of column (typically six) and the count
-        ## how many columns we should drop at the end in order to find the
-        ## second as the last
-        ## 
-        n <- lengths(out)
-        n <- sort(n)[round(length(n) / 2)] ## "median" without using 'stats'
-        drop <- n - 2L
-        out <- lapply(out, FUN = function(x) rev(x)[-seq_len(drop)][1])
-        out <- unlist(out, use.names = FALSE)
-        if (debug) {
-          cat("Extracted: ", paste(sQuote(out), collapse = ", "), "\n", sep = "")
+          mdebug("Trimmed:")
+          mprint(out)
         }
+        out <- out[, 2]
+        out <- grep("^[[:digit:]]+$", out, value = TRUE)	
+        if (debug) mdebugf("Extracted: %s", commaq(out))
         out <- as.integer(out)
-        if (debug) {
-          cat("Parsed: ", paste(sQuote(out), collapse = ", "), "\n", sep = "")
-        }
+        if (debug) mdebugf("Parsed: %s", commaq(out))
         out <- (out == pid)
-        if (debug) {
-          cat("Equals PID: ", paste(out, collapse = ", "), "\n", sep = "")
-        }
+        if (debug) mdebugf("Equals PID: %s", commaq(out))
         any(out)
       }, error = function(ex) NA)
       if (isTRUE(res)) return(res)
@@ -182,7 +176,7 @@ pid_exists <- local({
 
   cache <- list()
 
-  function(pid, debug = getOption2("parallelly.debug", FALSE)) {
+  function(pid, debug = getOption("parallelly.debug", FALSE)) {
     stop_if_not(is.numeric(pid), length(pid) == 1L, is.finite(pid), pid > 0L)
 
     pid_check <- cache$pid_check
